<?php

namespace Netzmacher\QuickShop\TCA;

use TYPO3\CMS\Core\Utility\VersionNumberUtility;

if( !defined( 'TYPO3_MODE' ) )
{
	die( 'Access denied.' );
}

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017-2019 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @author      Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package     TYPO3
 * @subpackage  quick_shop
 * @version     8.14.0
 * @since       7.0.2
 */
class Palettes
{

	/**
	 * @access public
	 * @param				array	$palettes: current palettes array
	 * @param				boolean $fal
	 * @return      array $palettes: extended palettes array
	 * @version     8.10.0
	 * @since       7.0.2
	 */
	static public function Image( $palettes, $fal = 1 )
	{
		switch( $fal )
		{
			case(false):
				return SELF::_ImageCSV( $palettes );
			case(true):
			default:
				return SELF::_ImageFAL( $palettes );
		}
	}

	/**
	 * @access private
	 * @param				array	$palettes: current palettes array
	 * @return      array	$palettes
	 * @version     8.14.0
	 * @since       8.14.0
	 */
	static public function Media( $palettes )
	{
		return ( array ) $palettes + array(
				'documents_appearance' => array(
						'showitem' => ''
						. 'documentslayout;LLL:EXT:quick_shop/Resources/Private/Language/tx_quickshop.xml:tca_phrase.documentslayout,'
						. 'documentssize;LLL:EXT:quick_shop/Resources/Private/Language/6x/cms/locallang_ttc.xlf:filelink_size_formlabel'
						,
						'canNotCollapse' => 1,
				),
				'documents_upload' => array(
						'showitem' => ''
						. 'documents_from_path;LLL:EXT:quick_shop/Resources/Private/Language/tx_quickshop.xml:tca_phrase.documents_from_path,'
						. '--linebreak--,'
						. 'documents,'
						,
						'canNotCollapse' => 1,
				),
		);
	}

	/**
	 * @return      string
	 * @access public
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static public function RTE()
	{
		$TYPO3_version = VersionNumberUtility::convertVersionNumberToInteger( TYPO3_version );

		switch( TRUE )
		{
			case ( $TYPO3_version < 8007000 ):
				return ';;;richtext[]:rte_transform[mode=ts]';
			case ( $TYPO3_version >= 8007000 ):
			default:
				return NULL;
		}
	}

	/**
	 * @return      array $palettes: extended palettes array
	 * @access public
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static public function TypePage()
	{
		return array(
				'showitem' => ''
				. 'type,page'
				,
				'canNotCollapse' => 1,
		);
	}

	/**
	 * @return      array $palettes: extended palettes array
	 * @access public
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static public function TypeUrl()
	{
		return array(
				'showitem' => ''
				. 'type,url'
				,
				'canNotCollapse' => 1,
		);
	}

	/**
	 * @access private
	 * @param				array	$palettes: current palettes array
	 * @return      string
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static private function _ImageCSV( $palettes )
	{
		$_pathLL = SELF::_pathLL();

		return ( array ) $palettes + array(
				'image_accessibility' => array(
						'showitem' => ''
						. 'imageseo;LLL:EXT:quick_shop/Resources/Private/Language/tx_quickshop.xml:tca_phrase.imageseo,'
						,
						'canNotCollapse' => 1,
				),
				'imageblock' => array(
						'showitem' => ''
						. 'imageorient;LLL:EXT:' . $_pathLL . ':imageorient_formlabel,'
						. 'imagecols;LLL:EXT:' . $_pathLL . ':imagecols_formlabel,'
						. '--linebreak--,'
						. 'image_noRows;LLL:EXT:' . $_pathLL . ':image_noRows_formlabel,'
						. 'imagecaption_position;LLL:EXT:' . $_pathLL . ':imagecaption_position_formlabel'
						,
						'canNotCollapse' => 1,
				),
				'imagefiles' => array(
						'showitem' => ''
						. 'image;LLL:EXT:' . $_pathLL . ':image_formlabel,'
						. 'imagecaption;LLL:EXT:' . $_pathLL . ':imagecaption_formlabel,'
						,
						'canNotCollapse' => 1,
				),
				'imagelinks' => array(
						'showitem' => ''
						. 'image_zoom;LLL:EXT:' . $_pathLL . ':image_zoom_formlabel,'
						. 'image_link;LLL:EXT:' . $_pathLL . ':image_link_formlabel'
						,
						'canNotCollapse' => 1,
				),
				'image_1stforlistonly' => array(
						'showitem' => 'image_1stforlistonly;LLL:EXT:quick_shop/Resources/Private/Language/tx_quickshop.xml:tca_phrase.image_1stforlistonly,',
						'canNotCollapse' => 1,
				),
				'image_settings' => array(
						'showitem' => ''
						. 'imagewidth;LLL:EXT:' . $_pathLL . ':imagewidth_formlabel,'
						. 'imageheight;LLL:EXT:' . $_pathLL . ':imageheight_formlabel,'
						. 'imageborder;LLL:EXT:' . $_pathLL . ':imageborder_formlabel,'
						. '--linebreak--,'
						. 'image_compression;LLL:EXT:' . $_pathLL . ':image_compression_formlabel,'
						. 'image_effects;LLL:EXT:' . $_pathLL . ':image_effects_formlabel,'
						. 'image_frames;LLL:EXT:' . $_pathLL . ':image_frames_formlabel'
						,
						'canNotCollapse' => 1,
				),
		);
	}

	/**
	 * @access private
	 * @param				array	$palettes: current palettes array
	 * @return      string
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static private function _ImageFAL( $palettes )
	{
		$TYPO3_version = VersionNumberUtility::convertVersionNumberToInteger( TYPO3_version );

		switch( TRUE )
		{
			case ( $TYPO3_version < 8007000 ):
				return SELF::_ImageFALCss( $palettes );
			case ( $TYPO3_version >= 8007000 ):
			default:
				return SELF::_ImageFALFluid( $palettes );
		}
	}

	/**
	 * @param				array	$palettes: current palettes array
	 * @access private
	 * @return      string
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static private function _ImageFALCss( $palettes )
	{
		$_pathLL = SELF::_pathLL();

		return ( array ) $palettes + array(
				// See: typo3/sysext/css_styled_content/Configuration/TCA/Overrides/tt_content.php
				'frames' => array(
						'showitem' => '
				layout;LLL:EXT:' . $_pathLL . ':layout_formlabel,
				spaceBefore;LLL:EXT:' . $_pathLL . ':spaceBefore_formlabel,
				spaceAfter;LLL:EXT:' . $_pathLL . ':spaceAfter_formlabel,
				section_frame;LLL:EXT:' . $_pathLL . ':section_frame_formlabel
			',
				),
				// See: typo3/sysext/css_styled_content/Configuration/TCA/Overrides/tt_content.php
				'imageblock' => array(
						'showitem' => '
				imageorient;LLL:EXT:' . $_pathLL . ':imageorient_formlabel,
				imagecols;LLL:EXT:' . $_pathLL . ':imagecols_formlabel,
				--linebreak--,
				image_noRows;LLL:EXT:' . $_pathLL . ':image_noRows_formlabel,
				imagecaption_position;LLL:EXT:' . $_pathLL . ':imagecaption_position_formlabel
			',
				),
				// See: typo3/sysext/frontend/Configuration/TCA/tt_content.php
				'imagelinks' => array(
						'showitem' => '
				image_zoom;LLL:EXT:' . $_pathLL . ':image_zoom_formlabel
			',
				),
				'image_1stforlistonly' => array(
						'showitem' => 'image_1stforlistonly;LLL:EXT:quick_shop/Resources/Private/Language/tx_quickshop.xml:tca_phrase.image_1stforlistonly,',
						'canNotCollapse' => 1,
				),
				// See: typo3/sysext/frontend/Configuration/TCA/tt_content.php
				'image_settings' => array(
						'showitem' => '
				imagewidth;LLL:EXT:' . $_pathLL . ':imagewidth_formlabel,
        imageheight;LLL:EXT:' . $_pathLL . ':imageheight_formlabel,
        imageborder;LLL:EXT:' . $_pathLL . ':imageborder_formlabel,
        --linebreak--,
        image_compression;LLL:EXT:' . $_pathLL . ':image_compression_formlabel,
        image_effects;LLL:EXT:' . $_pathLL . ':image_effects_formlabel,
			',
				),
		);
	}

	/**
	 * @param				array	$palettes: current palettes array
	 * @access private
	 * @return      string
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static private function _ImageFALFluid( $palettes )
	{
		return ( array ) $palettes + array(
				'appearanceLinks' => $GLOBALS[ 'TCA' ][ 'tt_content' ][ 'palettes' ][ 'appearanceLinks' ],
				'frames' => $GLOBALS[ 'TCA' ][ 'tt_content' ][ 'palettes' ][ 'frames' ],
				'gallerySettings' => $GLOBALS[ 'TCA' ][ 'tt_content' ][ 'palettes' ][ 'gallerySettings' ],
				'imagelinks' => $GLOBALS[ 'TCA' ][ 'tt_content' ][ 'palettes' ][ 'imagelinks' ],
				'image_1stforlistonly' => array(
						'showitem' => 'image_1stforlistonly;LLL:EXT:quick_shop/Resources/Private/Language/tx_quickshop.xml:tca_phrase.image_1stforlistonly,',
						'canNotCollapse' => 1,
				),
				'mediaAdjustments' => $GLOBALS[ 'TCA' ][ 'tt_content' ][ 'palettes' ][ 'mediaAdjustments' ],
		);
	}

	/**
	 * @access private
	 * @return      string
	 * @version     8.10.0
	 * @since       8.10.0
	 */
	static private function _pathLL()
	{
		$TYPO3_version = VersionNumberUtility::convertVersionNumberToInteger( TYPO3_version );

		switch( TRUE )
		{
			case ( $TYPO3_version < 7006000 ):
				return 'quick_shop/Resources/Private/Language/6x/cms/locallang_ttc.xlf';
			case ( $TYPO3_version >= 7006000 ):
			default:
				return 'frontend/Resources/Private/Language/locallang_ttc.xlf';
		}
	}

}
