<?php

namespace Netzmacher\QuickShop\Utility\Update;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017-2019 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 *
 * @author  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package TYPO3
 * @subpackage quick_shop
 * @version     8.13.4
 * @since       7.0.1
 */
class ScriptDatabase
{

	private $DBmapping = array();
	private $devDry = FALSE; // for development only. Live modus: FALSE
	private $ll = 'LLL:EXT:quick_shop/Resources/Private/Language/Update/scriptDB.xml:';

	private function actionDBUpdate()
	{
		// #i0263, 181025, dwildt, 1-/2+
		//$tables = array_map( trim, ( array ) \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'tables' ) );
		$tables = array_map( $callback, ( array ) \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'tables' ) );
		unset( $callback );

		switch( TRUE )
		{
			case(empty( $tables )):
				return ''
								. '<legend>' . $GLOBALS[ 'LANG' ]->sL( $this->ll . 'formDBHeader' ) . '</legend>'
								. $this->prompt( 'promptNoTableSelected', 'information' )
								. $this->form( NULL, NULL, 'formReloadButton' )
				;
			default:
				return $this->actionDBUpdateTables( $tables );
		}
	}

	private function actionDBUpdateTables( $tables )
	{
		$lines = array();
		$this->initDBmapping();
		foreach( $tables as $table => $csvFields )
		{
			if( !in_array( $table, array_keys( $this->DBmapping ) ) )
			{
				continue;
			}
			$lines[] = $this->actionDBUpdateTablesFields( $table, $csvFields );
		}
		$line = implode( ',', $lines );
		return $line;
	}

	private function actionDBUpdateTablesFields( $table, $csvFields )
	{
		$successTablefields = array();
		$fieldsDefinition = $this->sqlFieldsDefinition( $table, $csvFields );
		$fields = explode( ',', $csvFields );
		foreach( $fields as $field )
		{
			if( !isset( $this->DBmapping[ $table ][ $field ] ) )
			{
				continue;
			}
			$this->sqlRemoveField( $table, $field );
			$this->sqlRenameField( $table, $field, $fieldsDefinition );
			$successTablefields[] = $table . '.' . $field . ' => ' . $this->DBmapping[ $table ][ $field ];
		}

		$content = ''
						. '<ul>'
						. '<li>'
						. implode( ', </li>' . PHP_EOL . '<li>', $successTablefields )
						. '</li>'
						. '</ul>'
		;

		return ''
						. $this->prompt( 'promptUpdateDB', 'ok' )
						. $this->form( 'formDBSuccessHeader', $content, 'formReloadButton' )
		;
	}

	private function form( $legend, $content, $button, $action = NULL )
	{
		if( $legend )
		{
			$legend = '<legend>' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $legend ) . '</legend>';
		}
		return '
      <form action="" method="POST">
        ' . $legend . '
        <fieldset>
          ' . $this->formInput( $action ) . '
          ' . $content . '
          ' . $this->formButton( $button ) . '
        </fieldset>
			</form>
      <p>&nbsp;</p>
      '
		;
	}

	private function formButton( $button )
	{
		return ''
						. '<input type="submit" class="button" value="' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $button ) . '">' . PHP_EOL
		;
	}

	private function formInput( $action = NULL )
	{
		$moduleToken = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'moduleToken' );

		if( $action )
		{
			$action = '<input type="hidden" name="action" value="' . $action . '" />' . PHP_EOL;
		}

		return ''
						. '<input type="hidden" name="moduleToken" value="' . $moduleToken . '" />' . PHP_EOL
						. '<input type="hidden" name="M" value="tools_ExtensionmanagerExtensionmanager" />' . PHP_EOL
						. '<input type="hidden" name="tx_extensionmanager_tools_extensionmanagerextensionmanager[extensionKey]" value="quick_shop" />' . PHP_EOL
						. '<input type="hidden" name="tx_extensionmanager_tools_extensionmanagerextensionmanager[action]" value="show" />' . PHP_EOL
						. '<input type="hidden" name="tx_extensionmanager_tools_extensionmanagerextensionmanager[controller]" value="UpdateScript" />' . PHP_EOL
						. $action
		;
	}

	private function formTablesList( $tables )
	{

		$input = NULL;

		foreach( $tables as $table => $fields )
		{
			$prompt = $this->formTablesListPrompt( $table, $fields );
			if( empty( $prompt ) )
			{
				continue;
			}

//      var_dump( __METHOD__, __LINE__, $table, $fields, $prompt );
//      die();
			$csvValues = implode( ',', array_keys( $fields ) );

			$input = ''
							. $input
							. '<input id="DB' . $table . '" type="checkbox" name="tables[' . $table . ']" value="' . $csvValues . '" />' . PHP_EOL
							. '<label for="DB' . $table . '">' . $table . '</label>' . PHP_EOL
							. '<br />' . PHP_EOL
							. $prompt
							. '<br />' . PHP_EOL
			;
		}

		switch( TRUE )
		{
			case(empty( $input )):
				$input = ''
								. $this->prompt( 'promptDBIsDone', 'ok' )
								. '<br />' . PHP_EOL
				;
				break;
			case(!empty( $input )):
			default:
				$input = ''
								. $this->prompt( 'promptTopDb', 'warning' )
								. $this->prompt( 'promptBackupDb', 'information' )
								. $input
								. '<br />' . PHP_EOL
				;
				break;
		}

		return $input;
	}

	private function formTablesListPrompt( $table, $fields )
	{

		if( empty( $fields ) )
		{
			return NULL;
		}

		$arrFields = array();
		foreach( $fields as $srcField => $destField )
		{
			$arrFields[] = $srcField . ' => ' . $destField;
		}
		$csvFields = implode( ', ', $arrFields );

		$prompt = ''
						. 'The fields of ' . $table . ' will renamed: ' . $csvFields . '. '
						. 'BE AWARE: the destination fields will removed before renaming!'
						. '<br />' . PHP_EOL
		;

		return $prompt;
	}

	private function getTablesForUpdate()
	{
		$this->initDBmapping();
		$tables = $this->setTablesForUpdate();
		return $tables;
	}

	private function initDBmapping()
	{
		$this->DBmapping = array(
				'tx_quickshop_products' => array(
						'caption' => 'imagecaption',
						'caption_position' => 'imagecaption_position',
				),
		);
	}

	/**
	 * Main function, returning the HTML content of the update script
	 *
	 * @return	string		HTML
	 * @version     7.0.1
	 * @since       7.0.1
	 */
	public function main()
	{
		$action = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'action' );

		switch( TRUE )
		{
			case( $action == 'updateDB'):
				$content = $this->updateDB();
				break;
			case(!empty( $action )):
				$content = NULL;
				break;
			default:
				$content = $this->menu();
				break;
		}

		if( !$this->devDry )
		{
			return $content;
		}

		var_dump( __METHOD__, __LINE__, $content );
		die();
	}

	/**
	 * Returns the menu
	 *
	 * @return	string		The menu
	 * @version     7.0.1
	 * @since       7.0.0
	 */
	private function menu()
	{
		$tables = $this->getTablesForUpdate();

		switch( TRUE )
		{
			case(empty( $tables )):
				return $this->promptDBisDone();
			case(!empty( $tables )):
			default:
				return $this->form( 'formDBHeader', $this->formTablesList( $tables ), 'formDBButton', 'updateDB' );
		}
	}

	private function prompt( $prompt, $severity = "information" )
	{
		// #i0262, 181025, dwildt, 4+
		$alertSeverity = $severity;
		switch( TRUE )
		{
			case( $alertSeverity == "information" ):
				$alertSeverity = "info";
				break;
			case( $alertSeverity == "ok" ):
				$alertSeverity = "success";
				break;
		}

		// #i0262, 181025, dwildt, ~
		return '
			<div class="alert alert-' . $alertSeverity . ' typo3-message message-' . $severity . '">
        <div class="message-header">' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $prompt . 'Header' ) . '</div>
        <div class="message-body">
          ' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $prompt . 'Prompt' ) . '
        </div>
      </div>' . PHP_EOL;
	}

	private function promptDBisDone()
	{
		$prompt = $this->prompt( 'promptDBIsDone', 'ok' );
		return $this->form( 'formDBHeader', $prompt, 'formReloadButton' );
	}

	private function setTablesForUpdate()
	{
		$tables = array();
		foreach( $this->DBmapping as $table => $fields )
		{
			//var_dump( __METHOD__, __LINE__, array_keys( $fields ) );
			$query = 'SHOW FIELDS FROM ' . $table;
			$res = $GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
			$this->sqlQueryErrorPrompt( $query, __METHOD__, __LINE__ );

			while( $row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_assoc( $res ) )
			{
				$field = $row[ 'Field' ];
				if( !in_array( $row[ 'Field' ], array_keys( $fields ) ) )
				{
					continue;
				}
				//var_dump( __METHOD__, __LINE__, $row, $row[ 'Field' ] );
				$tables[ $table ][ $field ] = $fields[ $field ];
			}
		}
		return $tables;
	}

	private function sqlFieldsDefinition( $table, $csvFields )
	{

		$query = 'SHOW FULL COLUMNS FROM ' . $table;
		$res = $GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->sqlQueryErrorPrompt( $query, __METHOD__, __LINE__ );

		$srceFields = explode( ',', $csvFields );
		$fieldsDefinition = array();

		foreach( $srceFields as $srceField )
		{
			$destField = $this->DBmapping[ $table ][ $srceField ];
			$destFields[ $destField ] = $srceField;
		}
//    var_dump(__METHOD__, __LINE__, $destFields);

		while( $row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_assoc( $res ) )
		{
			$destField = $row[ 'Field' ];
			if( !in_array( $destField, array_keys( $destFields ) ) )
			{
				continue;
			}
			$srceField = $destFields[ $destField ];
			$fieldsDefinition[ $srceField ] = $row;
		}
//    var_dump(__METHOD__, __LINE__, $fieldsDefinition);
		return $fieldsDefinition;
	}

	private function sqlQueryErrorPrompt( $query, $method, $line )
	{
		$error = $GLOBALS[ 'TYPO3_DB' ]->sql_error();

		if( !$error )
		{
			return;
		}

		$prompt = '<h1>Organiser: SQL-Error</h1>' . PHP_EOL
						. '<p>' . $error . '</p>' . PHP_EOL
						. '<p>' . $query . '</p>' . PHP_EOL
						. '<p>' . PHP_EOL
						. 'Sorry for the trouble.<br />' . PHP_EOL
						. 'TYPO3 Caddy<br />' . PHP_EOL
						. $method . ' (' . $line . ')'
		;
		die( $prompt );
	}

	private function sqlRemoveField( $table, $field )
	{
		$destField = $this->DBmapping[ $table ][ $field ];

		$query = 'ALTER TABLE  ' . $table . ' DROP ' . $destField;

		if( !$this->devDry )
		{
			$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
			$this->sqlQueryErrorPrompt( $query, __METHOD__, __LINE__ );
			return;
		}

		var_dump( __METHOD__, __LINE__, $query );
	}

	private function sqlRenameField( $table, $srceField, $fieldsDefinition )
	{
		$destField = $this->DBmapping[ $table ][ $srceField ];
//    var_dump(__METHOD__, __LINE__, $srceField, $destField);
		$query = 'ALTER TABLE  ' . $table . ' CHANGE'
						. ' ' . $srceField
						. ' ' . $destField
						. ' ' . $fieldsDefinition[ $srceField ][ 'Type' ]
						. $this->sqlRenameFieldNull( $srceField, $fieldsDefinition )
						. $this->sqlRenameFieldDefault( $srceField, $fieldsDefinition )
		;
		if( !$this->devDry )
		{
			$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
			$this->sqlQueryErrorPrompt( $query, __METHOD__, __LINE__ );
			return;
		}

		var_dump( __METHOD__, __LINE__, $query );
	}

	private function sqlRenameFieldDefault( $srceField, $fieldsDefinition )
	{
		switch( TRUE )
		{
			// #i0088, 170807, dwildt, 3-
//			case( $fieldsDefinition[ $srceField ][ 'Default' ] === NULL ):
//			default:
//				return " DEFAULT ''";
			case(!empty( $fieldsDefinition[ $srceField ][ 'Default' ] ) ):
				// #i0088, 170807, dwildt, 1-
				//default:
				return ' DEFAULT ' . $fieldsDefinition[ $srceField ][ 'Default' ];
			// #i0088, 170807, dwildt, 1+
			case( $fieldsDefinition[ $srceField ][ 'Default' ] === NULL ):
			case( $fieldsDefinition[ $srceField ][ 'Default' ] === '' ):
			default:
				return " DEFAULT ''";
		}
	}

	private function sqlRenameFieldNull( $srceField, $fieldsDefinition )
	{
		if( $fieldsDefinition[ $srceField ][ 'Null' ] == 'Yes' )
		{
			return ' NULL';
		}

		return '';
		return ' NOT NULL';
	}

	private function updateDB()
	{
		switch( TRUE )
		{
			case( \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'action' ) == 'updateDB'):
				return $this->actionDBUpdate();
			default:
				return '<a href="' . \TYPO3\CMS\Core\Utility\GeneralUtility::linkThisScript( array( 'do_update' => '', 'func' => '' ) ) . '">'
								. $GLOBALS[ 'LANG' ]->sL( $this->ll . 'back' )
								. '</a>'
								. '<br>';
		}
	}

}
